@@ -272,6 +272,19 @@ macro_rules! exclude_impls {
     ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
 }
 
+/// Indicates that a C++ type is not to be generated by autocxx in this case,
+/// but instead should refer to some pre-existing Rust type.
+///
+/// The syntax is:
+/// `extern_cpp_type!("CppNameGoesHere", path::to::rust::type)`
+///
+/// A directive to be included inside
+/// [include_cpp] - see [include_cpp] for general information.
+#[macro_export]
+macro_rules! extern_cpp_type {
+    ($($tt:tt)*) => { $crate::usage!{$($tt)*} };
+}
+
 /// Deprecated - use [`extern_rust_type`] instead.
 #[macro_export]
 #[deprecated]
@@ -305,6 +318,8 @@ macro_rules! usage {
     };
 }
 
+use std::pin::Pin;
+
 #[doc(hidden)]
 pub use autocxx_macro::include_cpp_impl;
 
@@ -447,6 +462,49 @@ pub trait PinMut<T>: AsRef<T> {
     fn pin_mut(&mut self) -> std::pin::Pin<&mut T>;
 }
 
+/// Provides utility functions to emplace any [`moveit::New`] into a
+/// [`cxx::UniquePtr`]. Automatically imported by the autocxx prelude
+/// and implemented by any (autocxx-related) [`moveit::New`].
+pub trait WithinUniquePtr {
+    type Inner: UniquePtrTarget + MakeCppStorage;
+    fn within_unique_ptr(self) -> cxx::UniquePtr<Self::Inner>;
+}
+
+/// Provides utility functions to emplace any [`moveit::New`] into a
+/// [`Box`]. Automatically imported by the autocxx prelude
+/// and implemented by any (autocxx-related) [`moveit::New`].
+pub trait WithinBox {
+    type Inner;
+    fn within_box(self) -> Pin<Box<Self::Inner>>;
+}
+
+use moveit::MakeCppStorage;
+use moveit::{Emplace, EmplaceUnpinned};
+
+impl<N, T> WithinUniquePtr for N
+where
+    N: New<Output = T>,
+    T: UniquePtrTarget + MakeCppStorage,
+{
+    type Inner = T;
+    fn within_unique_ptr(self) -> cxx::UniquePtr<T> {
+        UniquePtr::emplace(self)
+    }
+}
+
+impl<N, T> WithinBox for N
+where
+    N: New<Output = T>,
+{
+    type Inner = T;
+    fn within_box(self) -> Pin<Box<T>> {
+        Box::emplace(self)
+    }
+}
+
+use cxx::memory::UniquePtrTarget;
+use cxx::UniquePtr;
+use moveit::New;
 pub use value_param::as_copy;
 pub use value_param::as_mov;
 pub use value_param::as_new;
@@ -472,9 +530,13 @@ pub mod prelude {
     pub use crate::include_cpp;
     pub use crate::PinMut;
     pub use crate::ValueParam;
+    pub use crate::WithinBox;
+    pub use crate::WithinUniquePtr;
     pub use cxx::UniquePtr;
     pub use moveit::moveit;
     pub use moveit::new::New;
+    pub use moveit::Emplace;
+    pub use moveit::EmplaceUnpinned;
 }
 
 /// Re-export moveit for ease of consumers.
